#ifndef _ABBusca_H_
#define _ABBusca_H_

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#define MAX(a,b) ((a > b)?a:b)
#define TAM_MAX 100

/* Arvore Binaria de Busca com valores menores na subarvore esquerda e
   maiores ou iguais na subarvore direita de cada no */

typedef struct NoABBusca {  /* Registro que representa n em rvore
			      binria */
  int info;                    
  struct NoABBusca *esq, *dir;
} NoABBusca, ABBusca;

/* cria no folha (insercao eh sempre de no folha) */

NoABBusca *CriaNovoNo(int info); 

/* Esta funo deve inserir o valor da varivel "valor" na rvore
   binria de busca representada pela varivel "ab". Note que valores
   iguais devem ser inseridos sempre  direita. */

void InsereValor(ABBusca **ab, int valor); 

/* Esta funo deve percorrer a rvore ab em pr-ordem e gerar uma
   cadeia de caracteres s com os valores visitados em pr-ordem
   separados por um espao em branco. */

void VisitaPreOrdem(ABBusca *ab, char **s); 

/* Esta funo deve percorrer a rvore ab em in-ordem e gerar uma
   cadeia de caracteres s com os valores visitados em in-ordem
   separados por um espao em branco. */

void VisitaInOrdem(ABBusca *ab, char **s); 

/* Esta funo deve percorrer a rvore ab em ps-ordem e gerar uma
   cadeia de caracteres s com os valores visitados em ps-ordem
   separados por um espao em branco. */

void VisitaPosOrdem(ABBusca *ab, char **s);  

/* Esta funo deve remover o valor da varivel "valor" da rvore
   binria de busca representada pela varivel "ab". Note que no caso
   de remoo de n de grau 2, seu valor deve ser substitudo pelo
   valor do menor sucessor (no a esquerda com menor valor da subarvore
   direita), e o no sucessor deve ser removido. Isso porque a arvore
   resolve a igualdade inserindo o no na subarvore direita. Este
   sucessor nao tem filho a esquerda, mas pode ter filho a
   direita. Portanto, o no pai do sucessor deve apontar para o seu
   filho a direita antes da remocao do sucessor. Se a igualdade fosse
   resolvida com insercao na subarvore esquerda, deveriamos procurar o
   maior antecessor para substituicao. Neste caso, ele nao teria filho
   a direita, mas poderia ter filho a esquerda e o seu no pai deveria
   apontar para o filho a esquerda antes da remocao. */

void RemoveValor(ABBusca **ab, int valor); 

/* Esta funo deve liberar todo espao de memria alocado para a
   rvore */

void DestroiABBusca(ABBusca **ab); 

/* Esta funo deve calcular e retornar o nmero de ns da rvore
   ab */

int  NumerodeNos(ABBusca *ab); 

/* Esta funo deve calcular e retornar a altura da rvore ab */

int  Altura(ABBusca *ab); 

/* Esta funo deve calcular os k maiores valores armazenados na
   rvore ab e retorn-los em ordem decrescente e separados por um
   espao em branco na cadeia de caracteres s */

void MaioresValores(ABBusca *ab, char **s, int *k); 

/* Esta funo deve calcular os k menores valores armazenados na
   rvore ab e retorn-los em ordem crescente e separados por um
   espao em branco na cadeia de caracteres s */

void MenoresValores(ABBusca *ab, char **s, int *k); 

/* Esta funo supe que a rvore ab armazena apenas valores no
   negativos. Ela deve calcular o maior valor K armazenado na rvore e
   criar um vetor de freqncia f de tamanho n=K+1. Depois ela deve
   calcular e armazenar em f[i] o nmero de ocorrncias do valor i na
   rvore ab. Por fim, ela deve retornar o vetor f e seu tamanho n. */

void Frequencia(ABBusca *ab, int **f, int *n); 

/* Esta funo deve remover da rvore ab todos os valores repetidos
   deixando apenas uma ocorrncia de cada valor. */

void EliminaRepetidos(ABBusca **ab); 

/* Imprime arvore binaria na tela onde h (altura do no) indica que ele
   sera precedido de h caracteres '-' antes de sua impressao. A funcao
   e chamada com h=0. */

void ImprimeABBusca(ABBusca *ab, int h, int altura);

/* Arvore Equilibrada: aquela que os numeros de nos das subarvores
   direita e esquerda nao diferem mais do que 1 para qualquer no. */

char Equilibrada(ABBusca *ab, int *N); 

#endif









